@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -75,13 +75,13 @@ impl FlickableData {
                 }
             }
             MouseEvent::MouseMoved { pos } => {
-                if inner.capture_events
+                let do_intercept = inner.capture_events
                     || inner.pressed_time.map_or(false, |pressed_time| {
                         crate::animations::current_tick() - pressed_time < DURATION_THRESHOLD
                             && (pos - inner.pressed_pos).square_length()
                                 > DISTANCE_THRESHOLD * DISTANCE_THRESHOLD
-                    })
-                {
+                    });
+                if do_intercept {
                     InputEventFilterResult::Intercept
                 } else if inner.pressed_time.is_some() {
                     InputEventFilterResult::ForwardAndInterceptGrab
