@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -14,6 +14,7 @@ LICENSE END */
 use crate::graphics::Point;
 use crate::item_tree::{ItemVisitorResult, VisitChildrenResult};
 use crate::items::{ItemRc, ItemRef, ItemWeak};
+use crate::window::WindowRc;
 use crate::Property;
 use crate::{component::ComponentRc, SharedString};
 use const_field_offset::FieldOffsets;
@@ -28,7 +29,7 @@ use std::rc::Rc;
 pub enum MouseEvent {
     /// The mouse was pressed
     MousePressed { pos: Point },
-    /// The mouse was relased
+    /// The mouse was released
     MouseReleased { pos: Point },
     /// The mouse position has changed
     MouseMoved { pos: Point },
@@ -70,7 +71,7 @@ impl MouseEvent {
 /// This value is returned by the `input_event` function of an Item
 /// to notify the run-time about how the event was handled and
 /// what the next steps are.
-/// See [`ItemVTable::input_event`].
+/// See [`crate::items::ItemVTable::input_event`].
 #[repr(C)]
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum InputEventResult {
@@ -91,21 +92,21 @@ impl Default for InputEventResult {
 
 /// This value is returned by the `input_event_filter_before_children` function, which
 /// can specify how to further process the event.
-/// See [`ItemVTable::input_event_filter_before_children`].
+/// See [`crate::items::ItemVTable::input_event_filter_before_children`].
 #[repr(C)]
 #[derive(Debug, Copy, Clone, Eq, PartialEq)]
 pub enum InputEventFilterResult {
-    /// The event is going to be forwarded to children, then the [`ItemVTable::input_event`]
+    /// The event is going to be forwarded to children, then the [`crate::items::ItemVTable::input_event`]
     /// function is called
     ForwardEvent,
-    /// The event will be forwarded to the children, but the [`ItemVTable::input_event`] is not
+    /// The event will be forwarded to the children, but the [`crate::items::ItemVTable::input_event`] is not
     /// going to be called for this item
     ForwardAndIgnore,
     /// Just like `ForwardEvent`, but even in the case the children grabs the mouse, this function
     /// Will still be called for further event.
     ForwardAndInterceptGrab,
     /// The Event will not be forwarded to children, if a children already had the grab, the
-    /// grab will be cancelled with a [`MouseEventType::MouseExit`] event
+    /// grab will be cancelled with a [`MouseEvent::MouseExit`] event
     Intercept,
 }
 
@@ -198,10 +199,10 @@ pub struct KeyboardModifiers {
     pub alt: bool,
     /// Indicates the control key on a keyboard.
     pub control: bool,
-    /// Indicates the shift key on a keyboard.
-    pub shift: bool,
     /// Indicates the logo key on macOS and the windows key on Windows.
     pub meta: bool,
+    /// Indicates the shift key on a keyboard.
+    pub shift: bool,
 }
 
 #[derive(Debug, Clone, PartialEq, strum_macros::EnumString, strum_macros::Display)]
@@ -224,10 +225,12 @@ impl Default for KeyEventType {
 #[derive(Debug, Clone, PartialEq, Default)]
 #[repr(C)]
 pub struct KeyEvent {
-    /// The unicode representation of the key pressed.
-    pub text: SharedString,
     /// The keyboard modifiers active at the time of the key press event.
     pub modifiers: KeyboardModifiers,
+    /// The unicode representation of the key pressed.
+    pub text: SharedString,
+
+    // note: this field is not exported in the .60 in the KeyEvent builtin struct
     /// Indicates whether the key was pressed or released
     pub event_type: KeyEventType,
 }
@@ -261,78 +264,101 @@ pub enum FocusEvent {
 /// The state which a window should hold for the mouse input
 #[derive(Default)]
 pub struct MouseInputState {
-    /// The stack of item which contain the mouse cursor (or grab)
-    item_stack: Vec<ItemWeak>,
+    /// The stack of item which contain the mouse cursor (or grab),
+    /// along with the last result from the input function
+    item_stack: Vec<(ItemWeak, InputEventFilterResult)>,
     /// true if the top item of the stack has the mouse grab
     grabbed: bool,
 }
 
-/// Process the `mouse_event` on the `component`, the `mouse_grabber_stack` is the prebious stack
+/// Try to handle the mouse grabber. Return true if the event has handled, or false otherwise
+fn handle_mouse_grab(
+    mouse_event: &MouseEvent,
+    window: &WindowRc,
+    mouse_input_state: &mut MouseInputState,
+) -> bool {
+    if !mouse_input_state.grabbed || mouse_input_state.item_stack.is_empty() {
+        return false;
+    };
+
+    let mut event = *mouse_event;
+    let mut intercept = false;
+    let mut invalid = false;
+
+    mouse_input_state.item_stack.retain(|it| {
+        if invalid {
+            return false;
+        }
+        let item = if let Some(item) = it.0.upgrade() {
+            item
+        } else {
+            invalid = true;
+            return false;
+        };
+        if intercept {
+            item.borrow().as_ref().input_event(MouseEvent::MouseExit, window, &item);
+            return false;
+        }
+        let g = item.borrow().as_ref().geometry();
+        event.translate(-g.origin.to_vector());
+
+        if it.1 == InputEventFilterResult::ForwardAndInterceptGrab
+            && item.borrow().as_ref().input_event_filter_before_children(event, window, &item)
+                == InputEventFilterResult::Intercept
+        {
+            intercept = true;
+        }
+        true
+    });
+    if invalid {
+        return false;
+    }
+
+    let grabber = mouse_input_state.item_stack.last().unwrap().0.upgrade().unwrap();
+    let input_result = grabber.borrow().as_ref().input_event(event, window, &grabber);
+    if input_result != InputEventResult::GrabMouse {
+        mouse_input_state.grabbed = false;
+        send_exit_events(&mouse_input_state, mouse_event.pos(), window);
+    }
+
+    true
+}
+
+fn send_exit_events(
+    mouse_input_state: &MouseInputState,
+    mut pos: Option<Point>,
+    window: &WindowRc,
+) {
+    for it in mouse_input_state.item_stack.iter() {
+        let item = if let Some(item) = it.0.upgrade() { item } else { break };
+        let g = item.borrow().as_ref().geometry();
+        let contains = pos.map_or(false, |p| g.contains(p));
+        if let Some(p) = pos.as_mut() {
+            *p -= g.origin.to_vector();
+        }
+        if !contains {
+            item.borrow().as_ref().input_event(MouseEvent::MouseExit, window, &item);
+        }
+    }
+}
+
+/// Process the `mouse_event` on the `component`, the `mouse_grabber_stack` is the previous stack
 /// of mouse grabber.
 /// Returns a new mouse grabber stack.
 pub fn process_mouse_input(
     component: ComponentRc,
     mouse_event: MouseEvent,
-    window: &crate::window::ComponentWindow,
+    window: &WindowRc,
     mut mouse_input_state: MouseInputState,
 ) -> MouseInputState {
-    'grab: loop {
-        if !mouse_input_state.grabbed || mouse_input_state.item_stack.is_empty() {
-            break 'grab;
-        };
-        let mut event = mouse_event.clone();
-        let mut intercept = false;
-        let mut invalid = false;
-
-        mouse_input_state.item_stack.retain(|it| {
-            if invalid {
-                return false;
-            }
-            let item = if let Some(item) = it.upgrade() {
-                item
-            } else {
-                invalid = true;
-                return false;
-            };
-            if intercept {
-                item.borrow().as_ref().input_event(MouseEvent::MouseExit, window, &item);
-                return false;
-            }
-            let g = item.borrow().as_ref().geometry();
-            event.translate(-g.origin.to_vector());
-
-            if item.borrow().as_ref().input_event_filter_before_children(event, window, &item)
-                == InputEventFilterResult::Intercept
-            {
-                intercept = true;
-            }
-            true
-        });
-        if invalid {
-            break 'grab;
-        }
-
-        let grabber = mouse_input_state.item_stack.last().unwrap().upgrade().unwrap();
-        return match grabber.borrow().as_ref().input_event(event, window, &grabber) {
-            InputEventResult::GrabMouse => mouse_input_state,
-            _ => Default::default(),
-        };
+    if handle_mouse_grab(&mouse_event, window, &mut mouse_input_state) {
+        return mouse_input_state;
     }
 
-    let mut pos = mouse_event.pos();
-    // Send the Exit event.
-    for it in mouse_input_state.item_stack.iter() {
-        let item = if let Some(item) = it.upgrade() { item } else { break };
-        let g = item.borrow().as_ref().geometry();
-        let contains = pos.map_or(false, |p| g.contains(p));
-        pos.as_mut().map(|p| *p -= g.origin.to_vector());
-        if !contains {
-            item.borrow().as_ref().input_event(MouseEvent::MouseExit, window, &item);
-        }
-    }
+    send_exit_events(&mouse_input_state, mouse_event.pos(), window);
 
     let mut result = MouseInputState::default();
-    type State = (Vector2D<f32>, Vec<ItemWeak>);
+    type State = (Vector2D<f32>, Vec<(ItemWeak, InputEventFilterResult)>);
     crate::item_tree::visit_items_with_post_visit(
         &component,
         crate::item_tree::TraversalOrder::FrontToBack,
@@ -346,34 +372,33 @@ pub fn process_mouse_input(
             let geom = geom.translate(*offset);
 
             let mut mouse_grabber_stack = mouse_grabber_stack.clone();
-            // FIXME: ideally we should add ourself to the stack only if InputEventFilterResult::ForwardAndInterceptGrab
-            // is used, but at the moment, we also use the mouse_grabber_stack to compute the offset
-            mouse_grabber_stack.push(item_rc.downgrade());
 
-            let post_visit_state = if mouse_event.pos().map_or(false, |p| geom.contains(p)) {
-                let mut event2 = mouse_event.clone();
+            let post_visit_state = if mouse_event.pos().map_or(false, |p| geom.contains(p))
+                || crate::item_rendering::is_clipping_item(item)
+            {
+                let mut event2 = mouse_event;
                 event2.translate(-geom.origin.to_vector());
-
-                match item.as_ref().input_event_filter_before_children(
-                    event2.clone(),
-                    window,
-                    &item_rc,
-                ) {
+                let filter_result =
+                    item.as_ref().input_event_filter_before_children(event2, window, &item_rc);
+                mouse_grabber_stack.push((item_rc.downgrade(), filter_result));
+                match filter_result {
                     InputEventFilterResult::ForwardAndIgnore => None,
                     InputEventFilterResult::ForwardEvent => {
-                        Some((event2, mouse_grabber_stack.clone(), item_rc.clone(), false))
+                        Some((event2, mouse_grabber_stack.clone(), item_rc, false))
                     }
                     InputEventFilterResult::ForwardAndInterceptGrab => {
-                        Some((event2, mouse_grabber_stack.clone(), item_rc.clone(), false))
+                        Some((event2, mouse_grabber_stack.clone(), item_rc, false))
                     }
                     InputEventFilterResult::Intercept => {
                         return (
                             ItemVisitorResult::Abort,
-                            Some((event2, mouse_grabber_stack.clone(), item_rc.clone(), true)),
+                            Some((event2, mouse_grabber_stack, item_rc, true)),
                         )
                     }
                 }
             } else {
+                mouse_grabber_stack
+                    .push((item_rc.downgrade(), InputEventFilterResult::ForwardAndIgnore));
                 None
             };
 
@@ -393,13 +418,17 @@ pub fn process_mouse_input(
                         result.grabbed = false;
                         return VisitChildrenResult::abort(item_rc.index(), 0);
                     }
-                    InputEventResult::EventIgnored => (),
+                    InputEventResult::EventIgnored => {
+                        return VisitChildrenResult::CONTINUE;
+                    }
                     InputEventResult::GrabMouse => {
-                        result.item_stack = mouse_grabber_stack.clone();
+                        result.item_stack = mouse_grabber_stack;
+                        result.item_stack.last_mut().unwrap().1 =
+                            InputEventFilterResult::ForwardAndInterceptGrab;
                         result.grabbed = true;
                         return VisitChildrenResult::abort(item_rc.index(), 0);
                     }
-                };
+                }
             }
             r
         },
