@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -35,25 +35,34 @@ impl CachedRenderingData {
     /// This function allows retrieving the backend specific per-item data cache, updating
     /// it if depending properties have changed. The supplied update_fn will be called when
     /// properties have changed or the cache is initialized the first time.
-    pub fn ensure_up_to_date<T: Clone>(
+    pub fn get_or_update<T: Clone>(
         &self,
-        cache: &mut RenderingCache<T>,
+        cache: &RefCell<RenderingCache<T>>,
         update_fn: impl FnOnce() -> T,
     ) -> T {
-        if self.cache_generation.get() == cache.generation() {
+        if self.cache_generation.get() == cache.borrow().generation()
+            && cache.borrow().contains(self.cache_index.get())
+        {
             let index = self.cache_index.get();
-            if let Some(existing_entry) = cache.get_mut(index) {
-                if let Some(data) =
-                    existing_entry.dependency_tracker.as_ref().evaluate_if_dirty(update_fn)
-                {
-                    existing_entry.data = data
-                }
-                return existing_entry.data.clone();
+            let tracker = cache.borrow_mut().get_mut(index).unwrap().dependency_tracker.take();
+
+            let maybe_new_data =
+                tracker.as_ref().and_then(|tracker| tracker.as_ref().evaluate_if_dirty(update_fn));
+
+            let mut cache = cache.borrow_mut();
+            let cache_entry = cache.get_mut(index).unwrap();
+            cache_entry.dependency_tracker = tracker;
+
+            if let Some(new_data) = maybe_new_data {
+                cache_entry.data = new_data
             }
+
+            return cache_entry.data.clone();
         }
-        self.cache_index.set(cache.insert(crate::graphics::CachedGraphicsData::new(update_fn)));
-        self.cache_generation.set(cache.generation());
-        cache.get(self.cache_index.get()).unwrap().data.clone()
+        let cache_entry = crate::graphics::CachedGraphicsData::new(update_fn);
+        self.cache_index.set(cache.borrow_mut().insert(cache_entry));
+        self.cache_generation.set(cache.borrow().generation());
+        cache.borrow().get(self.cache_index.get()).unwrap().data.clone()
     }
 
     /// This function can be used to remove an entry from the rendering cache for a given item, if it
@@ -70,6 +79,13 @@ impl CachedRenderingData {
     }
 }
 
+/// Return true if the item might be a clipping item
+pub(crate) fn is_clipping_item(item: Pin<ItemRef>) -> bool {
+    //(FIXME: there should be some flag in the vtable instead of downcasting)
+    ItemRef::downcast_pin::<Flickable>(item).is_some()
+        || ItemRef::downcast_pin::<Clip>(item).is_some()
+}
+
 /// Renders the tree of items that component holds, using the specified renderer. Rendering is done
 /// relative to the specified origin.
 pub fn render_component_items(
@@ -95,10 +111,9 @@ pub fn render_component_items(
             let item_origin = item_geometry.origin;
 
             // Don't render items that are clipped, with the exception of the Clip or Flickable since
-            // they themself clip their content.  (FIXME: there should be some flag in the vtable instead of downcasting)
+            // they themselves clip their content.
             if !renderer.borrow().get_current_clip().intersects(&item_geometry)
-                && ItemRef::downcast_pin::<Flickable>(item).is_none()
-                && ItemRef::downcast_pin::<Clip>(item).is_none()
+                && !is_clipping_item(item)
             {
                 renderer.borrow_mut().translate(item_origin.x, item_origin.y);
                 return (ItemVisitorResult::Continue(()), ());
@@ -155,7 +170,7 @@ pub trait ItemRenderer {
     /// Draw a pixmap in position indicated by the `pos`.
     /// The pixmap will be taken from cache if the cache is valid, otherwise, update_fn will be called
     /// with a callback that need to be called once with `fn (width, height, data)` where data are the
-    /// argb premultiplied pixel values
+    /// RGBA premultiplied pixel values
     fn draw_cached_pixmap(
         &mut self,
         item_cache: &CachedRenderingData,
