@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -26,7 +26,7 @@ When adding an item or a property, it needs to be kept in sync with different pl
 
 use crate::component::ComponentVTable;
 use crate::graphics::PathDataIterator;
-use crate::graphics::{Brush, Color, PathData, Rect};
+use crate::graphics::{Brush, Color, PathData, Point, Rect};
 use crate::input::{
     FocusEvent, InputEventFilterResult, InputEventResult, KeyEvent, KeyEventResult, KeyEventType,
     MouseEvent,
@@ -35,7 +35,7 @@ use crate::item_rendering::CachedRenderingData;
 use crate::layout::{LayoutInfo, Orientation};
 #[cfg(feature = "rtti")]
 use crate::rtti::*;
-use crate::window::ComponentWindow;
+use crate::window::WindowRc;
 use crate::{Callback, Property, SharedString};
 use const_field_offset::FieldOffsets;
 use core::pin::Pin;
@@ -54,6 +54,7 @@ type ItemRendererRef<'a> = &'a mut dyn crate::item_rendering::ItemRenderer;
 /// Workarounds for cbindgen
 pub type VoidArg = ();
 type KeyEventArg = (KeyEvent,);
+type PointArg = (Point,);
 
 #[macro_export]
 macro_rules! declare_item_vtable {
@@ -78,7 +79,7 @@ pub struct ItemVTable {
     /// This function is called by the run-time after the memory for the item
     /// has been allocated and initialized. It will be called before any user specified
     /// bindings are set.
-    pub init: extern "C" fn(core::pin::Pin<VRef<ItemVTable>>, window: &ComponentWindow),
+    pub init: extern "C" fn(core::pin::Pin<VRef<ItemVTable>>, window: &WindowRc),
 
     /// Returns the geometry of this item (relative to its parent item)
     pub geometry: extern "C" fn(core::pin::Pin<VRef<ItemVTable>>) -> Rect,
@@ -93,7 +94,7 @@ pub struct ItemVTable {
     pub layouting_info: extern "C" fn(
         core::pin::Pin<VRef<ItemVTable>>,
         orientation: Orientation,
-        window: &ComponentWindow,
+        window: &WindowRc,
     ) -> LayoutInfo,
 
     /// Event handler for mouse and touch event. This function is called before being called on children.
@@ -103,7 +104,7 @@ pub struct ItemVTable {
     pub input_event_filter_before_children: extern "C" fn(
         core::pin::Pin<VRef<ItemVTable>>,
         MouseEvent,
-        window: &ComponentWindow,
+        window: &WindowRc,
         self_rc: &ItemRc,
     ) -> InputEventFilterResult,
 
@@ -111,17 +112,17 @@ pub struct ItemVTable {
     pub input_event: extern "C" fn(
         core::pin::Pin<VRef<ItemVTable>>,
         MouseEvent,
-        window: &ComponentWindow,
+        window: &WindowRc,
         self_rc: &ItemRc,
     ) -> InputEventResult,
 
     pub focus_event:
-        extern "C" fn(core::pin::Pin<VRef<ItemVTable>>, &FocusEvent, window: &ComponentWindow),
+        extern "C" fn(core::pin::Pin<VRef<ItemVTable>>, &FocusEvent, window: &WindowRc),
 
     pub key_event: extern "C" fn(
         core::pin::Pin<VRef<ItemVTable>>,
         &KeyEvent,
-        window: &ComponentWindow,
+        window: &WindowRc,
     ) -> KeyEventResult,
 
     pub render: extern "C" fn(core::pin::Pin<VRef<ItemVTable>>, backend: &mut ItemRendererRef),
@@ -202,7 +203,7 @@ pub struct Rectangle {
 }
 
 impl Item for Rectangle {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -211,7 +212,7 @@ impl Item for Rectangle {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
@@ -219,7 +220,7 @@ impl Item for Rectangle {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -228,17 +229,17 @@ impl Item for Rectangle {
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
         (*backend).draw_rectangle(self)
@@ -273,7 +274,7 @@ pub struct BorderRectangle {
 }
 
 impl Item for BorderRectangle {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -282,7 +283,7 @@ impl Item for BorderRectangle {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
@@ -290,7 +291,7 @@ impl Item for BorderRectangle {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -299,17 +300,17 @@ impl Item for BorderRectangle {
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
         (*backend).draw_border_rectangle(self)
@@ -337,10 +338,10 @@ pub struct TouchArea {
     pub width: Property<f32>,
     pub height: Property<f32>,
     pub enabled: Property<bool>,
-    /// FIXME: We should anotate this as an "output" property.
+    /// FIXME: We should annotate this as an "output" property.
     pub pressed: Property<bool>,
     pub has_hover: Property<bool>,
-    /// FIXME: there should be just one property for the point istead of two.
+    /// FIXME: there should be just one property for the point instead of two.
     /// Could even be merged with pressed in a Property<Option<Point>> (of course, in the
     /// implementation item only, for the compiler it would stay separate properties)
     pub pressed_x: Property<f32>,
@@ -354,7 +355,7 @@ pub struct TouchArea {
 }
 
 impl Item for TouchArea {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -363,7 +364,7 @@ impl Item for TouchArea {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo::default()
     }
@@ -371,7 +372,7 @@ impl Item for TouchArea {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         event: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         if !self.enabled() {
@@ -388,7 +389,7 @@ impl Item for TouchArea {
     fn input_event(
         self: Pin<&Self>,
         event: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         if matches!(event, MouseEvent::MouseExit) {
@@ -397,8 +398,10 @@ impl Item for TouchArea {
         if !self.enabled() {
             return InputEventResult::EventIgnored;
         }
-        let result = if matches!(event, MouseEvent::MouseReleased { .. }) {
-            Self::FIELD_OFFSETS.clicked.apply_pin(self).call(&());
+        let result = if let MouseEvent::MouseReleased { pos, .. } = event {
+            if euclid::rect(0., 0., self.width(), self.height()).contains(pos) {
+                Self::FIELD_OFFSETS.clicked.apply_pin(self).call(&());
+            }
             InputEventResult::EventAccepted
         } else {
             InputEventResult::GrabMouse
@@ -422,11 +425,11 @@ impl Item for TouchArea {
         result
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, _backend: &mut ItemRendererRef) {}
 }
@@ -474,7 +477,7 @@ pub struct FocusScope {
 }
 
 impl Item for FocusScope {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -483,7 +486,7 @@ impl Item for FocusScope {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo::default()
     }
@@ -491,7 +494,7 @@ impl Item for FocusScope {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardEvent
@@ -500,21 +503,19 @@ impl Item for FocusScope {
     fn input_event(
         self: Pin<&Self>,
         event: MouseEvent,
-        window: &ComponentWindow,
+        window: &WindowRc,
         self_rc: &ItemRc,
     ) -> InputEventResult {
         /*if !self.enabled() {
             return InputEventResult::EventIgnored;
         }*/
-        if matches!(event, MouseEvent::MousePressed { .. }) {
-            if !self.has_focus() {
-                window.set_focus_item(self_rc);
-            }
+        if matches!(event, MouseEvent::MousePressed { .. }) && !self.has_focus() {
+            window.clone().set_focus_item(self_rc);
         }
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, event: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, event: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         let r = match event.event_type {
             KeyEventType::KeyPressed => {
                 Self::FIELD_OFFSETS.key_pressed.apply_pin(self).call(&(event.clone(),))
@@ -529,7 +530,7 @@ impl Item for FocusScope {
         }
     }
 
-    fn focus_event(self: Pin<&Self>, event: &FocusEvent, _window: &ComponentWindow) {
+    fn focus_event(self: Pin<&Self>, event: &FocusEvent, _window: &WindowRc) {
         match event {
             FocusEvent::FocusIn | FocusEvent::WindowReceivedFocus => {
                 self.has_focus.set(true);
@@ -566,10 +567,11 @@ pub struct Clip {
     pub border_radius: Property<f32>,
     pub border_width: Property<f32>,
     pub cached_rendering_data: CachedRenderingData,
+    pub clip: Property<bool>,
 }
 
 impl Item for Clip {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -578,42 +580,51 @@ impl Item for Clip {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
 
     fn input_event_filter_before_children(
         self: Pin<&Self>,
-        _: MouseEvent,
-        _window: &ComponentWindow,
+        event: MouseEvent,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
+        if let Some(pos) = event.pos() {
+            if self.clip()
+                && (pos.x < 0. || pos.y < 0. || pos.x > self.width() || pos.y > self.height())
+            {
+                return InputEventFilterResult::Intercept;
+            }
+        }
         InputEventFilterResult::ForwardAndIgnore
     }
 
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
-        let geometry = self.geometry();
-        (*backend).combine_clip(
-            euclid::rect(0., 0., geometry.width(), geometry.height()),
-            self.border_radius(),
-            self.border_width(),
-        )
+        if self.clip() {
+            let geometry = self.geometry();
+            (*backend).combine_clip(
+                euclid::rect(0., 0., geometry.width(), geometry.height()),
+                self.border_radius(),
+                self.border_width(),
+            )
+        }
     }
 }
 
@@ -641,7 +652,7 @@ pub struct Opacity {
 }
 
 impl Item for Opacity {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -650,7 +661,7 @@ impl Item for Opacity {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
@@ -658,7 +669,7 @@ impl Item for Opacity {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -667,17 +678,17 @@ impl Item for Opacity {
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
         backend.apply_opacity(self.opacity());
@@ -709,7 +720,7 @@ pub struct Rotate {
 }
 
 impl Item for Rotate {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(0., 0., 0., 0.)
@@ -718,7 +729,7 @@ impl Item for Rotate {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
@@ -726,7 +737,7 @@ impl Item for Rotate {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -735,17 +746,17 @@ impl Item for Rotate {
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
         (*backend).translate(self.origin_x(), self.origin_y());
@@ -802,7 +813,7 @@ pub struct Path {
 }
 
 impl Item for Path {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -811,34 +822,41 @@ impl Item for Path {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo::default()
     }
 
     fn input_event_filter_before_children(
         self: Pin<&Self>,
-        _: MouseEvent,
-        _window: &ComponentWindow,
+        event: MouseEvent,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
+        if let Some(pos) = event.pos() {
+            if self.clip()
+                && (pos.x < 0. || pos.y < 0. || pos.x > self.width() || pos.y > self.height())
+            {
+                return InputEventFilterResult::Intercept;
+            }
+        }
         InputEventFilterResult::ForwardAndIgnore
     }
 
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
         let clip = self.clip();
@@ -908,7 +926,7 @@ pub struct Flickable {
 }
 
 impl Item for Flickable {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -917,7 +935,7 @@ impl Item for Flickable {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
@@ -925,9 +943,14 @@ impl Item for Flickable {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         event: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
+        if let Some(pos) = event.pos() {
+            if pos.x < 0. || pos.y < 0. || pos.x > self.width() || pos.y > self.height() {
+                return InputEventFilterResult::Intercept;
+            }
+        }
         if !self.interactive() && !matches!(event, MouseEvent::MouseWheel { .. }) {
             return InputEventFilterResult::ForwardAndIgnore;
         }
@@ -937,20 +960,28 @@ impl Item for Flickable {
     fn input_event(
         self: Pin<&Self>,
         event: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         if !self.interactive() && !matches!(event, MouseEvent::MouseWheel { .. }) {
             return InputEventResult::EventIgnored;
         }
+        if let Some(pos) = event.pos() {
+            if matches!(event, MouseEvent::MouseWheel { .. } | MouseEvent::MousePressed { .. })
+                && (pos.x < 0. || pos.y < 0. || pos.x > self.width() || pos.y > self.height())
+            {
+                return InputEventResult::EventIgnored;
+            }
+        }
+
         self.data.handle_mouse(self, event)
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
         let geometry = self.geometry();
@@ -970,7 +1001,7 @@ declare_item_vtable! {
 pub use crate::SharedVector;
 
 #[repr(C)]
-/// Wraps the internal datastructure for the Flickable
+/// Wraps the internal data structure for the Flickable
 pub struct FlickableDataBox(core::ptr::NonNull<crate::flickable::FlickableData>);
 
 impl Default for FlickableDataBox {
@@ -994,10 +1025,16 @@ impl core::ops::Deref for FlickableDataBox {
     }
 }
 
+/// # Safety
+/// This must be called using a non-null pointer pointing to a chunk of memory big enough to
+/// hold a FlickableDataBox
 #[no_mangle]
 pub unsafe extern "C" fn sixtyfps_flickable_data_init(data: *mut FlickableDataBox) {
     std::ptr::write(data, FlickableDataBox::default());
 }
+
+/// # Safety
+/// This must be called using a non-null pointer pointing to an initialized FlickableDataBox
 #[no_mangle]
 pub unsafe extern "C" fn sixtyfps_flickable_data_free(data: *mut FlickableDataBox) {
     std::ptr::drop_in_place(data);
@@ -1020,19 +1057,20 @@ pub struct PropertyAnimation {
 #[repr(C)]
 #[derive(FieldOffsets, Default, SixtyFPSElement)]
 #[pin]
-pub struct Window {
+pub struct WindowItem {
     pub width: Property<f32>,
     pub height: Property<f32>,
     pub background: Property<Color>,
     pub title: Property<SharedString>,
+    pub icon: Property<crate::graphics::Image>,
     pub default_font_family: Property<SharedString>,
     pub default_font_size: Property<f32>,
     pub default_font_weight: Property<i32>,
     pub cached_rendering_data: CachedRenderingData,
 }
 
-impl Item for Window {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+impl Item for WindowItem {
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(0., 0., self.width(), self.height())
@@ -1041,7 +1079,7 @@ impl Item for Window {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo::default()
     }
@@ -1049,7 +1087,7 @@ impl Item for Window {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -1058,22 +1096,22 @@ impl Item for Window {
     fn input_event(
         self: Pin<&Self>,
         _event: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, _backend: &mut ItemRendererRef) {}
 }
 
-impl Window {
+impl WindowItem {
     /// Returns the font properties that can be used as defaults for child items
     pub fn default_font_properties(self: Pin<&Self>) -> crate::graphics::FontRequest {
         crate::graphics::FontRequest {
@@ -1106,13 +1144,13 @@ impl Window {
     }
 }
 
-impl ItemConsts for Window {
+impl ItemConsts for WindowItem {
     const cached_rendering_data_offset: const_field_offset::FieldOffset<Self, CachedRenderingData> =
         Self::FIELD_OFFSETS.cached_rendering_data.as_unpinned_projection();
 }
 
 declare_item_vtable! {
-    fn sixtyfps_get_WindowVTable() -> WindowVTable for Window
+    fn sixtyfps_get_WindowItemVTable() -> WindowItemVTable for WindowItem
 }
 
 /// The implementation of the `BoxShadow` element
@@ -1135,7 +1173,7 @@ pub struct BoxShadow {
 }
 
 impl Item for BoxShadow {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
@@ -1144,7 +1182,7 @@ impl Item for BoxShadow {
     fn layouting_info(
         self: Pin<&Self>,
         _orientation: Orientation,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
     ) -> LayoutInfo {
         LayoutInfo { stretch: 1., ..LayoutInfo::default() }
     }
@@ -1152,7 +1190,7 @@ impl Item for BoxShadow {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -1161,17 +1199,17 @@ impl Item for BoxShadow {
     fn input_event(
         self: Pin<&Self>,
         _event: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut ItemRendererRef) {
         (*backend).draw_box_shadow(self)
