@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -14,7 +14,7 @@ When adding an item or a property, it needs to be kept in sync with different pl
 Lookup the [`crate::items`] module documentation.
 */
 
-use super::{Item, ItemConsts, ItemRc, VoidArg};
+use super::{Item, ItemConsts, ItemRc, PointArg, VoidArg};
 use crate::graphics::{Brush, Color, FontRequest, Rect};
 use crate::input::{
     FocusEvent, InputEventResult, KeyEvent, KeyEventResult, KeyEventType, KeyboardModifiers,
@@ -25,7 +25,7 @@ use crate::item_rendering::{CachedRenderingData, ItemRenderer};
 use crate::layout::{LayoutInfo, Orientation};
 #[cfg(feature = "rtti")]
 use crate::rtti::*;
-use crate::window::ComponentWindow;
+use crate::window::WindowRc;
 use crate::{Callback, Property, SharedString};
 use const_field_offset::FieldOffsets;
 use core::pin::Pin;
@@ -112,34 +112,39 @@ pub struct Text {
 }
 
 impl Item for Text {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(
-        self: Pin<&Self>,
-        orientation: Orientation,
-        window: &ComponentWindow,
-    ) -> LayoutInfo {
-        let font_metrics = window.0.font_metrics(
-            &self.cached_rendering_data,
-            &|| self.unresolved_font_request(),
-            Self::FIELD_OFFSETS.text.apply_pin(self),
-        );
-        let text = self.text();
-        let implicit_size = font_metrics.text_size(&text, None);
+    fn layouting_info(self: Pin<&Self>, orientation: Orientation, window: &WindowRc) -> LayoutInfo {
+        let implicit_size = |max_width| {
+            window.text_size(
+                &self.cached_rendering_data,
+                &|| self.unresolved_font_request(),
+                self.text().as_str(),
+                max_width,
+            )
+        };
 
         // Stretch uses `round_layout` to explicitly align the top left and bottom right of layout nodes
         // to pixel boundaries. To avoid rounding down causing the minimum width to become so little that
         // letters will be cut off, apply the ceiling here.
         match orientation {
             Orientation::Horizontal => {
+                let implicit_size = implicit_size(None);
                 let min = match self.overflow() {
-                    TextOverflow::elide => {
-                        implicit_size.width.min(font_metrics.text_size("…", None).width)
-                    }
+                    TextOverflow::elide => implicit_size.width.min(
+                        window
+                            .text_size(
+                                &self.cached_rendering_data,
+                                &|| self.unresolved_font_request(),
+                                "…",
+                                None,
+                            )
+                            .width,
+                    ),
                     TextOverflow::clip => match self.wrap() {
                         TextWrap::no_wrap => implicit_size.width,
                         TextWrap::word_wrap => 0.,
@@ -153,8 +158,8 @@ impl Item for Text {
             }
             Orientation::Vertical => {
                 let h = match self.wrap() {
-                    TextWrap::no_wrap => implicit_size.height,
-                    TextWrap::word_wrap => font_metrics.text_size(&text, Some(self.width())).height,
+                    TextWrap::no_wrap => implicit_size(None).height,
+                    TextWrap::word_wrap => implicit_size(Some(self.width())).height,
                 }
                 .ceil();
                 LayoutInfo { min: h, preferred: h, ..LayoutInfo::default() }
@@ -165,7 +170,7 @@ impl Item for Text {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardAndIgnore
@@ -174,17 +179,17 @@ impl Item for Text {
     fn input_event(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventResult {
         InputEventResult::EventIgnored
     }
 
-    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, _: &KeyEvent, _window: &WindowRc) -> KeyEventResult {
         KeyEventResult::EventIgnored
     }
 
-    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &ComponentWindow) {}
+    fn focus_event(self: Pin<&Self>, _: &FocusEvent, _window: &WindowRc) {}
 
     fn render(self: Pin<&Self>, backend: &mut &mut dyn ItemRenderer) {
         (*backend).draw_text(self)
@@ -242,6 +247,7 @@ pub struct TextInput {
     pub selection_background_color: Property<Color>,
     pub horizontal_alignment: Property<TextHorizontalAlignment>,
     pub vertical_alignment: Property<TextVerticalAlignment>,
+    pub wrap: Property<TextWrap>,
     pub letter_spacing: Property<f32>,
     pub x: Property<f32>,
     pub y: Property<f32>,
@@ -254,40 +260,55 @@ pub struct TextInput {
     pub has_focus: Property<bool>,
     pub enabled: Property<bool>,
     pub accepted: Callback<VoidArg>,
+    pub cursor_position_changed: Callback<PointArg>,
     pub edited: Callback<VoidArg>,
     pub pressed: std::cell::Cell<bool>,
+    pub single_line: Property<bool>,
     pub cached_rendering_data: CachedRenderingData,
 }
 
 impl Item for TextInput {
-    fn init(self: Pin<&Self>, _window: &ComponentWindow) {}
+    fn init(self: Pin<&Self>, _window: &WindowRc) {}
 
     // FIXME: width / height.  or maybe it doesn't matter?  (
     fn geometry(self: Pin<&Self>) -> Rect {
         euclid::rect(self.x(), self.y(), self.width(), self.height())
     }
 
-    fn layouting_info(
-        self: Pin<&Self>,
-        orientation: Orientation,
-        window: &ComponentWindow,
-    ) -> LayoutInfo {
-        let font_metrics = window.0.font_metrics(
-            &self.cached_rendering_data,
-            &|| self.unresolved_font_request(),
-            Self::FIELD_OFFSETS.text.apply_pin(self),
-        );
-        let size = font_metrics.text_size("********************", None).ceil();
+    fn layouting_info(self: Pin<&Self>, orientation: Orientation, window: &WindowRc) -> LayoutInfo {
+        let implicit_size = |max_width| {
+            let text = self.text();
+            window.text_size(
+                &self.cached_rendering_data,
+                &|| self.unresolved_font_request(),
+                if text.is_empty() { "*" } else { text.as_str() },
+                max_width,
+            )
+        };
 
+        // Stretch uses `round_layout` to explicitly align the top left and bottom right of layout nodes
+        // to pixel boundaries. To avoid rounding down causing the minimum width to become so little that
+        // letters will be cut off, apply the ceiling here.
         match orientation {
-            Orientation::Horizontal => LayoutInfo {
-                min: size.width,
-                preferred: size.width,
-                stretch: 1.,
-                ..LayoutInfo::default()
-            },
+            Orientation::Horizontal => {
+                let implicit_size = implicit_size(None);
+                let min = match self.wrap() {
+                    TextWrap::no_wrap => implicit_size.width,
+                    TextWrap::word_wrap => 0.,
+                };
+                LayoutInfo {
+                    min: min.ceil(),
+                    preferred: implicit_size.width.ceil(),
+                    ..LayoutInfo::default()
+                }
+            }
             Orientation::Vertical => {
-                LayoutInfo { min: size.height, preferred: size.height, ..LayoutInfo::default() }
+                let h = match self.wrap() {
+                    TextWrap::no_wrap => implicit_size(None).height,
+                    TextWrap::word_wrap => implicit_size(Some(self.width())).height,
+                }
+                .ceil();
+                LayoutInfo { min: h, preferred: h, ..LayoutInfo::default() }
             }
         }
     }
@@ -295,7 +316,7 @@ impl Item for TextInput {
     fn input_event_filter_before_children(
         self: Pin<&Self>,
         _: MouseEvent,
-        _window: &ComponentWindow,
+        _window: &WindowRc,
         _self_rc: &ItemRc,
     ) -> InputEventFilterResult {
         InputEventFilterResult::ForwardEvent
@@ -304,27 +325,20 @@ impl Item for TextInput {
     fn input_event(
         self: Pin<&Self>,
         event: MouseEvent,
-        window: &ComponentWindow,
+        window: &WindowRc,
         self_rc: &ItemRc,
     ) -> InputEventResult {
         if !self.enabled() {
             return InputEventResult::EventIgnored;
         }
-
-        let text = self.text();
-        let font_metrics = window.0.font_metrics(
-            &self.cached_rendering_data,
-            &|| self.unresolved_font_request(),
-            Self::FIELD_OFFSETS.text.apply_pin(self),
-        );
         match event {
             MouseEvent::MousePressed { pos } => {
-                let clicked_offset = font_metrics.text_offset_for_x_position(&text, pos.x) as i32;
+                let clicked_offset = window.text_input_byte_offset_for_position(self, pos) as i32;
                 self.as_ref().pressed.set(true);
                 self.as_ref().anchor_position.set(clicked_offset);
-                self.as_ref().cursor_position.set(clicked_offset);
+                self.set_cursor_position(clicked_offset, window);
                 if !self.has_focus() {
-                    window.set_focus_item(self_rc);
+                    window.clone().set_focus_item(self_rc);
                 }
             }
             MouseEvent::MouseReleased { .. } | MouseEvent::MouseExit => {
@@ -333,8 +347,8 @@ impl Item for TextInput {
             MouseEvent::MouseMoved { pos } => {
                 if self.as_ref().pressed.get() {
                     let clicked_offset =
-                        font_metrics.text_offset_for_x_position(&text, pos.x) as i32;
-                    self.as_ref().cursor_position.set(clicked_offset);
+                        window.text_input_byte_offset_for_position(self, pos) as i32;
+                    self.set_cursor_position(clicked_offset, window);
                 }
             }
             MouseEvent::MouseWheel { .. } => return InputEventResult::EventIgnored,
@@ -342,7 +356,7 @@ impl Item for TextInput {
         InputEventResult::EventAccepted
     }
 
-    fn key_event(self: Pin<&Self>, event: &KeyEvent, window: &ComponentWindow) -> KeyEventResult {
+    fn key_event(self: Pin<&Self>, event: &KeyEvent, window: &WindowRc) -> KeyEventResult {
         use std::convert::TryFrom;
 
         if !self.enabled() {
@@ -367,38 +381,43 @@ impl Item for TextInput {
                     } else if keycode == InternalKeyCode::Delete {
                         TextInput::delete_char(self, window);
                         return KeyEventResult::EventAccepted;
-                    } else if keycode == InternalKeyCode::Return {
+                    } else if keycode == InternalKeyCode::Return && self.single_line() {
                         Self::FIELD_OFFSETS.accepted.apply_pin(self).call(&());
                         return KeyEventResult::EventAccepted;
                     }
                 }
 
                 // Only insert/interpreter non-control character strings
-                if event.text.is_empty() || event.text.as_str().chars().any(|ch| ch.is_control()) {
+                if event.text.is_empty()
+                    || event.text.as_str().chars().any(|ch| ch.is_control() && ch != '\n')
+                {
                     return KeyEventResult::EventIgnored;
                 }
                 if event.modifiers.control {
-                    if event.text == "c" {
+                    if event.text == "a" {
+                        self.select_all(window);
+                        return KeyEventResult::EventAccepted;
+                    } else if event.text == "c" {
                         self.copy();
                         return KeyEventResult::EventAccepted;
                     } else if event.text == "v" {
-                        self.paste();
+                        self.paste(window);
                         return KeyEventResult::EventAccepted;
                     }
                     return KeyEventResult::EventIgnored;
                 }
-                self.delete_selection();
+                self.delete_selection(window);
 
                 let mut text: String = self.text().into();
 
                 // FIXME: respect grapheme boundaries
-                let insert_pos = self.cursor_position() as usize;
+                let insert_pos = self.selection_anchor_and_cursor().1;
                 text.insert_str(insert_pos, &event.text);
 
                 self.as_ref().text.set(text.into());
                 let new_cursor_pos = (insert_pos + event.text.len()) as i32;
-                self.as_ref().cursor_position.set(new_cursor_pos);
                 self.as_ref().anchor_position.set(new_cursor_pos);
+                self.set_cursor_position(new_cursor_pos, window);
 
                 // Keep the cursor visible when inserting text. Blinking should only occur when
                 // nothing is entered or the cursor isn't moved.
@@ -412,7 +431,7 @@ impl Item for TextInput {
         }
     }
 
-    fn focus_event(self: Pin<&Self>, event: &FocusEvent, window: &ComponentWindow) {
+    fn focus_event(self: Pin<&Self>, event: &FocusEvent, window: &WindowRc) {
         match event {
             FocusEvent::FocusIn | FocusEvent::WindowReceivedFocus => {
                 self.has_focus.set(true);
@@ -440,8 +459,11 @@ impl ItemConsts for TextInput {
 enum TextCursorDirection {
     Forward,
     Backward,
+    PreviousCharacter, // breaks grapheme boundaries, so only used by delete-previous-char
     StartOfLine,
     EndOfLine,
+    StartOfText,
+    EndOfText,
 }
 
 impl std::convert::TryFrom<InternalKeyCode> for TextCursorDirection {
@@ -474,7 +496,7 @@ impl From<KeyboardModifiers> for AnchorMode {
 }
 
 impl TextInput {
-    fn show_cursor(&self, window: &ComponentWindow) {
+    fn show_cursor(&self, window: &WindowRc) {
         window.set_cursor_blink_binding(&self.cursor_visible);
     }
 
@@ -486,26 +508,26 @@ impl TextInput {
         self: Pin<&Self>,
         direction: TextCursorDirection,
         anchor_mode: AnchorMode,
-        window: &ComponentWindow,
+        window: &WindowRc,
     ) -> bool {
         let text = self.text();
-        if text.len() == 0 {
+        if text.is_empty() {
             return false;
         }
 
-        let last_cursor_pos = self.cursor_position() as usize;
+        let last_cursor_pos = (self.cursor_position() as usize).max(0).min(text.len());
+
+        let mut grapheme_cursor =
+            unicode_segmentation::GraphemeCursor::new(last_cursor_pos, text.len(), true);
 
         let new_cursor_pos = match direction {
             TextCursorDirection::Forward => {
-                let mut i = last_cursor_pos;
-                loop {
-                    i = i.checked_add(1).unwrap_or_default().min(text.len());
-                    if text.is_char_boundary(i) {
-                        break i;
-                    }
-                }
+                grapheme_cursor.next_boundary(&text, 0).ok().flatten().unwrap_or_else(|| text.len())
             }
             TextCursorDirection::Backward => {
+                grapheme_cursor.prev_boundary(&text, 0).ok().flatten().unwrap_or(0)
+            }
+            TextCursorDirection::PreviousCharacter => {
                 let mut i = last_cursor_pos;
                 loop {
                     i = i.checked_sub(1).unwrap_or_default();
@@ -514,18 +536,20 @@ impl TextInput {
                     }
                 }
             }
+            // FIXME: StartOfLine and EndOfLine should respect line boundaries
             TextCursorDirection::StartOfLine => 0,
             TextCursorDirection::EndOfLine => text.len(),
+            TextCursorDirection::StartOfText => 0,
+            TextCursorDirection::EndOfText => text.len(),
         };
 
-        self.as_ref().cursor_position.set(new_cursor_pos as i32);
-
         match anchor_mode {
             AnchorMode::KeepAnchor => {}
             AnchorMode::MoveAnchor => {
                 self.as_ref().anchor_position.set(new_cursor_pos as i32);
             }
         }
+        self.set_cursor_position(new_cursor_pos as i32, window);
 
         // Keep the cursor visible when moving. Blinking should only occur when
         // nothing is entered or the cursor isn't moved.
@@ -534,26 +558,35 @@ impl TextInput {
         new_cursor_pos != last_cursor_pos
     }
 
-    fn delete_char(self: Pin<&Self>, window: &ComponentWindow) {
+    fn set_cursor_position(self: Pin<&Self>, new_position: i32, window: &WindowRc) {
+        self.cursor_position.set(new_position);
+        if new_position >= 0 {
+            let pos = window.text_input_position_for_byte_offset(self, new_position as usize);
+            Self::FIELD_OFFSETS.cursor_position_changed.apply_pin(self).call(&(pos,));
+        }
+    }
+
+    fn delete_char(self: Pin<&Self>, window: &WindowRc) {
         if !self.has_selection() {
             self.move_cursor(TextCursorDirection::Forward, AnchorMode::KeepAnchor, window);
         }
-        self.delete_selection();
+        self.delete_selection(window);
     }
 
-    fn delete_previous(self: Pin<&Self>, window: &ComponentWindow) {
+    fn delete_previous(self: Pin<&Self>, window: &WindowRc) {
         if self.has_selection() {
-            self.delete_selection();
+            self.delete_selection(window);
             return;
         }
-        if self.move_cursor(TextCursorDirection::Backward, AnchorMode::MoveAnchor, window) {
+        if self.move_cursor(TextCursorDirection::PreviousCharacter, AnchorMode::MoveAnchor, window)
+        {
             self.delete_char(window);
         }
     }
 
-    fn delete_selection(self: Pin<&Self>) {
+    fn delete_selection(self: Pin<&Self>, window: &WindowRc) {
         let text: String = self.text().into();
-        if text.len() == 0 {
+        if text.is_empty() {
             return;
         }
 
@@ -563,15 +596,18 @@ impl TextInput {
         }
 
         let text = [text.split_at(anchor).0, text.split_at(cursor).1].concat();
-        self.cursor_position.set(anchor as i32);
-        self.anchor_position.set(anchor as i32);
         self.text.set(text.into());
+        self.anchor_position.set(anchor as i32);
+        self.set_cursor_position(anchor as i32, window);
         Self::FIELD_OFFSETS.edited.apply_pin(self).call(&());
     }
 
+    // Avoid accessing self.cursor_position()/self.anchor_position() directly, always
+    // use this bounds-checking function.
     pub fn selection_anchor_and_cursor(self: Pin<&Self>) -> (usize, usize) {
-        let cursor_pos = self.cursor_position().max(0);
-        let anchor_pos = self.anchor_position().max(0);
+        let max_pos = self.text().len() as i32;
+        let cursor_pos = self.cursor_position().max(0).min(max_pos);
+        let anchor_pos = self.anchor_position().max(0).min(max_pos);
 
         if anchor_pos > cursor_pos {
             (cursor_pos as _, anchor_pos as _)
@@ -591,26 +627,37 @@ impl TextInput {
         text.split_at(anchor).1.split_at(cursor - anchor).0.to_string()
     }
 
-    fn insert(self: Pin<&Self>, text_to_insert: &str) {
-        self.delete_selection();
+    fn insert(self: Pin<&Self>, text_to_insert: &str, window: &WindowRc) {
+        self.delete_selection(window);
         let mut text: String = self.text().into();
         let cursor_pos = self.selection_anchor_and_cursor().1;
-        text.insert_str(cursor_pos, text_to_insert);
+        if text_to_insert.contains('\n') && self.single_line() {
+            text.insert_str(cursor_pos, &text_to_insert.replace('\n', " "));
+        } else {
+            text.insert_str(cursor_pos, text_to_insert);
+        }
         let cursor_pos = cursor_pos + text_to_insert.len();
-        self.cursor_position.set(cursor_pos as i32);
-        self.anchor_position.set(cursor_pos as i32);
         self.text.set(text.into());
+        self.anchor_position.set(cursor_pos as i32);
+        self.set_cursor_position(cursor_pos as i32, window);
         Self::FIELD_OFFSETS.edited.apply_pin(self).call(&());
     }
 
-    fn copy(self: Pin<&Self>) {
-        crate::backend::instance().map(|backend| backend.set_clipboard_text(self.selected_text()));
+    fn select_all(self: Pin<&Self>, window: &WindowRc) {
+        self.move_cursor(TextCursorDirection::StartOfText, AnchorMode::MoveAnchor, window);
+        self.move_cursor(TextCursorDirection::EndOfText, AnchorMode::KeepAnchor, window);
     }
 
-    fn paste(self: Pin<&Self>) {
+    fn copy(self: Pin<&Self>) {
+        if let Some(backend) = crate::backend::instance() {
+            backend.set_clipboard_text(self.selected_text());
+        }
+    }
+
+    fn paste(self: Pin<&Self>, window: &WindowRc) {
         if let Some(text) = crate::backend::instance().and_then(|backend| backend.clipboard_text())
         {
-            self.insert(&text);
+            self.insert(&text, window);
         }
     }
 
