@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -9,6 +9,8 @@
 LICENSE END */
 //! Runtime support for layouts.
 
+// cspell:ignore coord
+
 use crate::{slice::Slice, SharedVector};
 
 /// Vertical or Horizontal orientation
@@ -23,22 +25,20 @@ type Coord = f32;
 
 /// The constraint that applies to an item
 // NOTE: when adding new fields, the C++ operator== also need updates
+// Also, the field needs to be in alphabetical order because how the generated code sort fields for struct
 #[repr(C)]
 #[derive(Clone, Copy, Debug, PartialEq)]
 pub struct LayoutInfo {
-    /// The minimum size for this item.
-    pub min: f32,
     /// The maximum size for the item.
     pub max: f32,
-
-    /// The minimum size in percentage of the parent (value between 0 and 100).
-    pub min_percent: f32,
     /// The maximum size in percentage of the parent (value between 0 and 100).
     pub max_percent: f32,
-
+    /// The minimum size for this item.
+    pub min: f32,
+    /// The minimum size in percentage of the parent (value between 0 and 100).
+    pub min_percent: f32,
     /// the preferred size
     pub preferred: f32,
-
     /// the  stretch factor
     pub stretch: f32,
 }
@@ -148,7 +148,7 @@ mod grid_internal {
     fn adjust_items<A: Adjust>(data: &mut [LayoutData], size_without_spacing: Coord) -> Option<()> {
         loop {
             let size_cannot_grow: Coord =
-                data.iter().filter(|it| !(A::can_grow(it) > 0.)).map(|it| it.size).sum();
+                data.iter().filter(|it| A::can_grow(it) <= 0.).map(|it| it.size).sum();
 
             let total_stretch: f32 =
                 data.iter().filter(|it| A::can_grow(it) > 0.).map(|it| it.stretch).sum();
@@ -206,6 +206,7 @@ mod grid_internal {
     }
 
     #[test]
+    #[allow(clippy::float_cmp)] // We want bit-wise equality here
     fn test_layout_items() {
         let my_items = &mut [
             LayoutData { min: 100., max: 200., pref: 100., stretch: 1., ..Default::default() },
@@ -228,6 +229,101 @@ mod grid_internal {
         assert_eq!(my_items[1].size, 100.);
         assert_eq!(my_items[2].size, 100.);
     }
+
+    /// Create a vector of LayoutData for an array of GridLayoutCellData
+    pub fn to_layout_data(
+        data: &[GridLayoutCellData],
+        spacing: Coord,
+        size: Option<Coord>,
+    ) -> Vec<LayoutData> {
+        let mut num = 0;
+        for cell in data {
+            num = num.max(cell.col_or_row + cell.span);
+        }
+        if num < 1 {
+            return Default::default();
+        }
+        let mut layout_data =
+            vec![grid_internal::LayoutData { stretch: 1., ..Default::default() }; num as usize];
+        let mut has_spans = false;
+        for cell in data {
+            let constraint = &cell.constraint;
+            let mut max = constraint.max;
+            if let Some(size) = size {
+                max = max.min(size * constraint.max_percent / 100.);
+            }
+            for c in 0..(cell.span as usize) {
+                let cdata = &mut layout_data[cell.col_or_row as usize + c];
+                cdata.max = cdata.max.min(max);
+            }
+            if cell.span == 1 {
+                let mut min = constraint.min;
+                if let Some(size) = size {
+                    min = min.max(size * constraint.min_percent / 100.);
+                }
+                let pref = constraint.preferred.min(max).max(min);
+                let cdata = &mut layout_data[cell.col_or_row as usize];
+                cdata.min = cdata.min.max(min);
+                cdata.pref = cdata.pref.max(pref);
+                cdata.stretch = cdata.stretch.min(constraint.stretch);
+            } else {
+                has_spans = true;
+            }
+        }
+        if has_spans {
+            // Adjust minimum sizes
+            for cell in data.iter().filter(|cell| cell.span > 1) {
+                let span_data = &mut layout_data
+                    [(cell.col_or_row as usize)..(cell.col_or_row + cell.span) as usize];
+                let mut min = cell.constraint.min;
+                if let Some(size) = size {
+                    min = min.max(size * cell.constraint.min_percent / 100.);
+                }
+                grid_internal::layout_items(span_data, 0., min, spacing);
+                for cdata in span_data {
+                    if cdata.min < cdata.size {
+                        cdata.min = cdata.size;
+                    }
+                }
+            }
+            // Adjust maximum sizes
+            for cell in data.iter().filter(|cell| cell.span > 1) {
+                let span_data = &mut layout_data
+                    [(cell.col_or_row as usize)..(cell.col_or_row + cell.span) as usize];
+                let mut max = cell.constraint.max;
+                if let Some(size) = size {
+                    max = max.min(size * cell.constraint.max_percent / 100.);
+                }
+                grid_internal::layout_items(span_data, 0., max, spacing);
+                for cdata in span_data {
+                    if cdata.max > cdata.size {
+                        cdata.max = cdata.size;
+                    }
+                }
+            }
+            // Adjust preferred sizes
+            for cell in data.iter().filter(|cell| cell.span > 1) {
+                let span_data = &mut layout_data
+                    [(cell.col_or_row as usize)..(cell.col_or_row + cell.span) as usize];
+                grid_internal::layout_items(span_data, 0., cell.constraint.preferred, spacing);
+                for cdata in span_data {
+                    cdata.pref = cdata.pref.max(cdata.size).min(cdata.max).max(cdata.min);
+                }
+            }
+            // Adjust stretches
+            for cell in data.iter().filter(|cell| cell.span > 1) {
+                let span_data = &mut layout_data
+                    [(cell.col_or_row as usize)..(cell.col_or_row + cell.span) as usize];
+                let total_stretch: f32 = span_data.iter().map(|c| c.stretch).sum();
+                if total_stretch > cell.constraint.stretch {
+                    for cdata in span_data {
+                        cdata.stretch *= cell.constraint.stretch / total_stretch;
+                    }
+                }
+            }
+        }
+        layout_data
+    }
 }
 
 #[repr(C)]
@@ -268,56 +364,22 @@ pub struct GridLayoutCellData {
     pub constraint: LayoutInfo,
 }
 
-/// return, an array which is of siz `data.cells.len() * 4` which for each cell we give the x, y, width, height
+/// return, an array which is of size `data.cells.len() * 2` which for each cell we give the pos, size
 pub fn solve_grid_layout(data: &GridLayoutData) -> SharedVector<Coord> {
-    let mut num = 0;
-    for cell in data.cells.iter() {
-        num = num.max(cell.col_or_row + cell.span);
-    }
+    let mut layout_data =
+        grid_internal::to_layout_data(data.cells.as_slice(), data.spacing, Some(data.size));
 
-    if num < 1 {
+    if layout_data.is_empty() {
         return Default::default();
     }
 
-    let mut layout_data =
-        vec![grid_internal::LayoutData { stretch: 1., ..Default::default() }; num as usize];
-
-    for cell in data.cells.iter() {
-        let cnstr = &cell.constraint;
-        let max = cnstr.max.min(data.size * cnstr.max_percent / 100.);
-        let min = cnstr.min.max(data.size * cnstr.min_percent / 100.) / (cell.span as f32);
-        let pref = (cnstr.preferred / cell.span as f32).min(max).max(min);
-
-        for c in 0..(cell.span as usize) {
-            let cdata = &mut layout_data[cell.col_or_row as usize + c];
-            cdata.max = cdata.max.min(max);
-            cdata.min = cdata.min.max(min);
-            cdata.pref = cdata.pref.max(pref);
-            cdata.stretch = cdata.stretch.min(cnstr.stretch);
-        }
-    }
-
-    // Normalize so that all the values are 1 or more
-    let normalize_stretch = |v: &mut Vec<grid_internal::LayoutData>| {
-        let mut small: Option<f32> = None;
-        v.iter().for_each(|x| {
-            if x.stretch > 0. {
-                small = Some(small.map(|y| y.min(x.stretch)).unwrap_or(x.stretch))
-            }
-        });
-        if small.unwrap_or(0.) < 1. {
-            v.iter_mut()
-                .for_each(|x| x.stretch = if let Some(s) = small { x.stretch / s } else { 1. })
-        }
-    };
-    normalize_stretch(&mut layout_data);
-
     grid_internal::layout_items(
         &mut layout_data,
         data.padding.begin,
         data.size - (data.padding.begin + data.padding.end),
         data.spacing,
     );
+
     let mut result = SharedVector::with_capacity(4 * data.cells.len());
     for cell in data.cells.iter() {
         let cdata = &layout_data[cell.col_or_row as usize];
@@ -331,30 +393,16 @@ pub fn solve_grid_layout(data: &GridLayoutData) -> SharedVector<Coord> {
     result
 }
 
-pub fn grid_layout_info<'a>(
-    cells: Slice<'a, GridLayoutCellData>,
+pub fn grid_layout_info(
+    cells: Slice<GridLayoutCellData>,
     spacing: Coord,
     padding: &Padding,
 ) -> LayoutInfo {
-    let mut num = 0;
-    for cell in cells.iter() {
-        num = num.max(cell.col_or_row + cell.span);
+    let layout_data = grid_internal::to_layout_data(cells.as_slice(), spacing, None);
+    if layout_data.is_empty() {
+        return Default::default();
     }
-
-    if num < 1 {
-        return LayoutInfo { max: 0., ..LayoutInfo::default() };
-    }
-
-    let mut layout_data = vec![grid_internal::LayoutData::default(); num as usize];
-    for cell in cells.iter() {
-        let cdata = &mut layout_data[cell.col_or_row as usize];
-        cdata.max = cdata.max.min(cell.constraint.max);
-        cdata.min = cdata.min.max(cell.constraint.min);
-        cdata.pref = cdata.pref.max(cell.constraint.preferred);
-        cdata.stretch = cdata.stretch.min(cell.constraint.stretch);
-    }
-
-    let spacing_w = spacing * (num - 1) as Coord + padding.begin + padding.end;
+    let spacing_w = spacing * (layout_data.len() - 1) as Coord + padding.begin + padding.end;
     let min = layout_data.iter().map(|data| data.min).sum::<Coord>() + spacing_w;
     let max = layout_data.iter().map(|data| data.max).sum::<Coord>() + spacing_w;
     let preferred = layout_data.iter().map(|data| data.pref).sum::<Coord>() + spacing_w;
@@ -402,6 +450,13 @@ pub struct BoxLayoutCellData {
 
 /// Solve a BoxLayout
 pub fn solve_box_layout(data: &BoxLayoutData, repeater_indexes: Slice<u32>) -> SharedVector<Coord> {
+    let mut result = SharedVector::<f32>::default();
+    result.resize(data.cells.len() * 2 + repeater_indexes.len(), 0.);
+
+    if data.cells.is_empty() {
+        return result;
+    }
+
     let mut layout_data: Vec<_> = data
         .cells
         .iter()
@@ -466,48 +521,46 @@ pub fn solve_box_layout(data: &BoxLayoutData, repeater_indexes: Slice<u32>) -> S
         }
     }
 
-    let mut result = SharedVector::<f32>::default();
-    result.resize(data.cells.len() * 2 + repeater_indexes.len(), 0.);
-    let res = result.as_slice_mut();
+    let res = result.make_mut_slice();
 
     // The index/2 in result in which we should add the next repeated item
-    let mut repeat_ofst =
+    let mut repeat_offset =
         res.len() / 2 - repeater_indexes.iter().skip(1).step_by(2).sum::<u32>() as usize;
     // The index/2  in repeater_indexes
     let mut next_rep = 0;
     // The index/2 in result in which we should add the next non-repeated item
-    let mut current_ofst = 0;
+    let mut current_offset = 0;
     for (idx, layout) in layout_data.iter().enumerate() {
         let o = loop {
             if let Some(nr) = repeater_indexes.get(next_rep * 2) {
                 let nr = *nr as usize;
                 if nr == idx {
                     for o in 0..2 {
-                        res[current_ofst * 2 + o] = (repeat_ofst * 2 + o) as _;
+                        res[current_offset * 2 + o] = (repeat_offset * 2 + o) as _;
                     }
-                    current_ofst += 1;
+                    current_offset += 1;
                 }
                 if idx >= nr {
                     if idx - nr == repeater_indexes[next_rep * 2 + 1] as usize {
                         next_rep += 1;
                         continue;
                     }
-                    repeat_ofst += 1;
-                    break repeat_ofst - 1;
+                    repeat_offset += 1;
+                    break repeat_offset - 1;
                 }
             }
-            current_ofst += 1;
-            break current_ofst - 1;
+            current_offset += 1;
+            break current_offset - 1;
         };
-        res[o * 2 + 0] = layout.pos;
+        res[o * 2] = layout.pos;
         res[o * 2 + 1] = layout.size;
     }
     result
 }
 
 /// Return the LayoutInfo for a BoxLayout with the given cells.
-pub fn box_layout_info<'a>(
-    cells: Slice<'a, BoxLayoutCellData>,
+pub fn box_layout_info(
+    cells: Slice<BoxLayoutCellData>,
     spacing: Coord,
     padding: &Padding,
     alignment: LayoutAlignment,
@@ -529,10 +582,7 @@ pub fn box_layout_info<'a>(
     LayoutInfo { min, max, min_percent: 0., max_percent: 100., preferred, stretch }
 }
 
-pub fn box_layout_info_ortho<'a>(
-    cells: Slice<'a, BoxLayoutCellData>,
-    padding: &Padding,
-) -> LayoutInfo {
+pub fn box_layout_info_ortho(cells: Slice<BoxLayoutCellData>, padding: &Padding) -> LayoutInfo {
     let count = cells.len();
     if count < 1 {
         return LayoutInfo { max: 0., ..LayoutInfo::default() };
@@ -603,15 +653,15 @@ pub fn solve_path_layout(data: &PathLayoutData, repeater_indexes: Slice<u32>) ->
 
     let mut result = SharedVector::<f32>::default();
     result.resize(data.item_count as usize * 2 + repeater_indexes.len(), 0.);
-    let res = result.as_slice_mut();
+    let res = result.make_mut_slice();
 
     // The index/2 in result in which we should add the next repeated item
-    let mut repeat_ofst =
+    let mut repeat_offset =
         res.len() / 2 - repeater_indexes.iter().skip(1).step_by(2).sum::<u32>() as usize;
     // The index/2  in repeater_indexes
     let mut next_rep = 0;
     // The index/2 in result in which we should add the next non-repeated item
-    let mut current_ofst = 0;
+    let mut current_offset = 0;
 
     'main_loop: while i < data.item_count {
         let mut current_length: f32 = 0.;
@@ -634,24 +684,24 @@ pub fn solve_path_layout(data: &PathLayoutData, repeater_indexes: Slice<u32>) ->
                         let nr = *nr;
                         if nr == i {
                             for o in 0..4 {
-                                res[current_ofst * 4 + o] = (repeat_ofst * 4 + o) as _;
+                                res[current_offset * 4 + o] = (repeat_offset * 4 + o) as _;
                             }
-                            current_ofst += 1;
+                            current_offset += 1;
                         }
                         if i >= nr {
                             if i - nr == repeater_indexes[next_rep * 2 + 1] {
                                 next_rep += 1;
                                 continue;
                             }
-                            repeat_ofst += 1;
-                            break repeat_ofst - 1;
+                            repeat_offset += 1;
+                            break repeat_offset - 1;
                         }
                     }
-                    current_ofst += 1;
-                    break current_ofst - 1;
+                    current_offset += 1;
+                    break current_offset - 1;
                 };
 
-                res[o * 2 + 0] = item_pos.x + data.x;
+                res[o * 2] = item_pos.x + data.x;
                 res[o * 2 + 1] = item_pos.y + data.y;
                 i += 1;
                 next_t += item_distance;
@@ -684,8 +734,8 @@ pub(crate) mod ffi {
     }
 
     #[no_mangle]
-    pub extern "C" fn sixtyfps_grid_layout_info<'a>(
-        cells: Slice<'a, GridLayoutCellData>,
+    pub extern "C" fn sixtyfps_grid_layout_info(
+        cells: Slice<GridLayoutCellData>,
         spacing: Coord,
         padding: &Padding,
     ) -> LayoutInfo {
@@ -703,8 +753,8 @@ pub(crate) mod ffi {
 
     #[no_mangle]
     /// Return the LayoutInfo for a BoxLayout with the given cells.
-    pub extern "C" fn sixtyfps_box_layout_info<'a>(
-        cells: Slice<'a, BoxLayoutCellData>,
+    pub extern "C" fn sixtyfps_box_layout_info(
+        cells: Slice<BoxLayoutCellData>,
         spacing: Coord,
         padding: &Padding,
         alignment: LayoutAlignment,
@@ -714,8 +764,8 @@ pub(crate) mod ffi {
 
     #[no_mangle]
     /// Return the LayoutInfo for a BoxLayout with the given cells.
-    pub extern "C" fn sixtyfps_box_layout_info_ortho<'a>(
-        cells: Slice<'a, BoxLayoutCellData>,
+    pub extern "C" fn sixtyfps_box_layout_info_ortho(
+        cells: Slice<BoxLayoutCellData>,
         padding: &Padding,
     ) -> LayoutInfo {
         super::box_layout_info_ortho(cells, padding)
