@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -32,7 +32,7 @@ pub struct ModelPeer {
 }
 
 /// Dispatch notifications from a [`Model`] to one or several [`ModelPeer`].
-/// Typically, you would want to put this in the implementaiton of the Model
+/// Typically, you would want to put this in the implementation of the Model
 #[derive(Default)]
 pub struct ModelNotify {
     inner: RefCell<weak_table::PtrWeakHashSet<Weak<RefCell<ModelPeerInner>>>>,
@@ -64,6 +64,68 @@ impl ModelNotify {
 }
 
 /// A Model is providing Data for the Repeater or ListView elements of the `.60` language
+///
+/// If the model can be changed, the type implementing the Model trait should holds
+/// a [`ModelNotify`], and is responsible to call functions on it to let the UI know that
+/// something has changed.
+///
+/// ## Example
+///
+/// As an example, let's see the implementation of [`VecModel`].
+///
+/// ```
+/// # use sixtyfps_corelib::model::{Model, ModelNotify, ModelPeer};
+/// pub struct VecModel<T> {
+///     // the backing data, stored in a `RefCell` as this model can be modified
+///     array: std::cell::RefCell<Vec<T>>,
+///     // the ModelNotify will allow to notify the UI that the model changes
+///     notify: ModelNotify,
+/// }
+///
+/// impl<T: Clone + 'static> Model for VecModel<T> {
+///     type Data = T;
+///
+///     fn row_count(&self) -> usize {
+///         self.array.borrow().len()
+///     }
+///
+///     fn row_data(&self, row: usize) -> Self::Data {
+///         self.array.borrow()[row].clone()
+///     }
+///
+///     fn set_row_data(&self, row: usize, data: Self::Data) {
+///         self.array.borrow_mut()[row] = data;
+///         // don't forget to call row_changed
+///         self.notify.row_changed(row);
+///     }
+///
+///     fn attach_peer(&self, peer: ModelPeer) {
+///         // simply forward to ModelNotify::attach
+///         self.notify.attach(peer);
+///     }
+///
+///     fn as_any(&self) -> &dyn core::any::Any {
+///         // a typical implementation just return `self`
+///         self
+///     }
+/// }
+///
+/// // when modifying the model, we call the corresponding function in
+/// // the ModelNotify
+/// impl<T> VecModel<T> {
+///     /// Add a row at the end of the model
+///     pub fn push(&self, value: T) {
+///         self.array.borrow_mut().push(value);
+///         self.notify.row_added(self.array.borrow().len() - 1, 1)
+///     }
+///
+///     /// Remove the row at the given index from the model
+///     pub fn remove(&self, index: usize) {
+///         self.array.borrow_mut().remove(index);
+///         self.notify.row_removed(index, 1)
+///     }
+/// }
+/// ```
 pub trait Model {
     /// The model data: A model is a set of row and each row has this data
     type Data;
@@ -71,20 +133,56 @@ pub trait Model {
     fn row_count(&self) -> usize;
     /// Returns the data for a particular row. This function should be called with `row < row_count()`.
     fn row_data(&self, row: usize) -> Self::Data;
-    /// Sets the data for a particular row. This function should be called with `row < row_count()`.
-    /// If the model cannot support data changes, then it is ok to do nothing (default implementation).
-    /// If the model can update the data, it should also call row_changed on its internal `ModelNotify`.
-    fn set_row_data(&self, _row: usize, _data: Self::Data) {}
-    /// Should forward to the internal [`ModelNotify::attach`]
+    /// Sets the data for a particular row.
+    ///
+    /// This function should be called with `row < row_count()`, otherwise the implementation can panic.
+    ///
+    /// If the model cannot support data changes, then it is ok to do nothing.
+    /// The default implementation will print a warning to stderr.
+    ///
+    /// If the model can update the data, it should also call [`ModelNofity::row_changed`] on its
+    /// internal [`ModelNotify`].
+    fn set_row_data(&self, _row: usize, _data: Self::Data) {
+        eprintln!(
+            "Model::set_row_data called on a model of type {} which does not re-implement this method. \
+            This happens when trying to modify a read-only model",
+            core::any::type_name::<Self>(),
+        );
+    }
+    /// The implementation should forward to [`ModelNotify::attach`]
     fn attach_peer(&self, peer: ModelPeer);
 
     /// Returns an iterator visiting all elements of the model.
-    fn iter<'a>(&'a self) -> ModelIterator<'a, Self::Data>
+    fn iter(&self) -> ModelIterator<Self::Data>
     where
         Self: Sized,
     {
         ModelIterator { model: self, row: 0 }
     }
+
+    /// Return something that can be downcast'ed (typically self)
+    ///
+    /// This is useful to get back to the actual model from a ModelHandle stored
+    /// in a component.
+    ///
+    /// ```
+    /// # use sixtyfps_corelib::model::*;
+    /// # use std::rc::Rc;
+    /// let vec_model = Rc::new(VecModel::from(vec![1i32, 2, 3]));
+    /// let handle = ModelHandle::from(vec_model as Rc<dyn Model<Data = i32>>);
+    /// // later:
+    /// handle.as_any().downcast_ref::<VecModel<i32>>().unwrap().push(4);
+    /// assert_eq!(handle.row_data(3), 4);
+    /// ```
+    ///
+    /// Note: the default implementation returns nothing interesting. this method should be
+    /// implemented by model implementation to return something useful. For example:
+    /// ```ignore
+    /// fn as_any(&self) -> &dyn core::any::Any { self }
+    /// ```
+    fn as_any(&self) -> &dyn core::any::Any {
+        &()
+    }
 }
 
 /// An iterator over the elements of a model.
@@ -115,7 +213,7 @@ impl<'a, T> Iterator for ModelIterator<'a, T> {
 
 impl<'a, T> ExactSizeIterator for ModelIterator<'a, T> {}
 
-/// A model backed by a SharedVector
+/// A model backed by a `Vec<T>`
 #[derive(Default)]
 pub struct VecModel<T> {
     array: RefCell<Vec<T>>,
@@ -128,7 +226,7 @@ impl<T: 'static> VecModel<T> {
     where
         T: Clone,
     {
-        ModelHandle(Some(Rc::<Self>::new(slice.iter().cloned().collect::<Vec<T>>().into())))
+        ModelHandle(Some(Rc::<Self>::new(slice.to_vec().into())))
     }
 
     /// Add a row at the end of the model
@@ -150,7 +248,7 @@ impl<T> From<Vec<T>> for VecModel<T> {
     }
 }
 
-impl<T: Clone> Model for VecModel<T> {
+impl<T: Clone + 'static> Model for VecModel<T> {
     type Data = T;
 
     fn row_count(&self) -> usize {
@@ -169,6 +267,10 @@ impl<T: Clone> Model for VecModel<T> {
     fn attach_peer(&self, peer: ModelPeer) {
         self.notify.attach(peer);
     }
+
+    fn as_any(&self) -> &dyn core::any::Any {
+        self
+    }
 }
 
 impl Model for usize {
@@ -185,6 +287,10 @@ impl Model for usize {
     fn attach_peer(&self, _peer: ModelPeer) {
         // The model is read_only: nothing to do
     }
+
+    fn as_any(&self) -> &dyn core::any::Any {
+        self
+    }
 }
 
 impl Model for bool {
@@ -203,6 +309,10 @@ impl Model for bool {
     fn attach_peer(&self, _peer: ModelPeer) {
         // The model is read_only: nothing to do
     }
+
+    fn as_any(&self) -> &dyn core::any::Any {
+        self
+    }
 }
 
 /// Properties of type array in the .60 language are represented as
@@ -233,7 +343,10 @@ impl<T> core::cmp::PartialEq for ModelHandle<T> {
             (None, None) => true,
             (None, Some(_)) => false,
             (Some(_), None) => false,
-            (Some(a), Some(b)) => Rc::ptr_eq(a, b),
+            (Some(a), Some(b)) => std::ptr::eq(
+                (&**a) as *const dyn Model<Data = T> as *const u8,
+                (&**b) as *const dyn Model<Data = T> as *const u8,
+            ),
         }
     }
 }
@@ -263,7 +376,13 @@ impl<T> Model for ModelHandle<T> {
     }
 
     fn attach_peer(&self, peer: ModelPeer) {
-        self.0.as_ref().map(|model| model.attach_peer(peer));
+        if let Some(model) = self.0.as_ref() {
+            model.attach_peer(peer);
+        }
+    }
+
+    fn as_any(&self) -> &dyn core::any::Any {
+        self.0.as_ref().map_or(&(), |model| model.as_any())
     }
 }
 
@@ -287,8 +406,8 @@ pub trait RepeatedComponent: crate::component::Component {
     }
 
     /// Returns what's needed to perform the layout if this component is in a box layout
-    fn box_layout_data<'a>(
-        self: Pin<&'a Self>,
+    fn box_layout_data(
+        self: Pin<&Self>,
         _orientation: Orientation,
     ) -> crate::layout::BoxLayoutCellData {
         crate::layout::BoxLayoutCellData::default()
@@ -325,7 +444,7 @@ impl<C: RepeatedComponent> Default for RepeaterInner<C> {
 
 impl<C: RepeatedComponent> Clone for RepeaterInner<C> {
     fn clone(&self) -> Self {
-        panic!("Clone is there so we can make_mut the RepeaterInner, to dissociate the weaks, but there should only be one inner")
+        panic!("Clone is there so we can make_mut the RepeaterInner, to dissociate the weak refs, but there should only be one inner")
     }
 }
 
@@ -396,7 +515,7 @@ pub struct Repeater<C: RepeatedComponent> {
     /// The model is changed. The inner RefCell make it possible to change the RepeaterInner when shared
     inner: RefCell<Rc<RefCell<RepeaterInner<C>>>>,
     model: Property<ModelHandle<C::Data>>,
-    /// Only used for the list view to track if the scrollbar has changed and item needs to be re_layouted
+    /// Only used for the list view to track if the scrollbar has changed and item needs to be layed out again.
     listview_geometry_tracker: crate::properties::PropertyTracker,
 }
 
@@ -417,7 +536,7 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
         let model = unsafe { self.map_unchecked(|s| &s.model) };
 
         if model.is_dirty() {
-            // Invalidate previuos weeks on the previous models
+            // Invalidate previous weeks on the previous models
             (*Rc::make_mut(&mut self.inner.borrow_mut()).get_mut()) = RepeaterInner::default();
             if let ModelHandle(Some(m)) = model.get() {
                 let peer: Rc<RefCell<dyn ViewAbstraction>> = self.inner.borrow().clone();
@@ -496,7 +615,8 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
         let listview_geometry_tracker =
             unsafe { self.map_unchecked(|s| &s.listview_geometry_tracker) };
         let init = &init;
-        if listview_geometry_tracker
+
+        let geometry_changed = listview_geometry_tracker
             .evaluate_if_dirty(|| {
                 let listview_height = listview_height.get();
                 // Compute the element height
@@ -511,15 +631,15 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
                     min_height.set(min_height.get().min(height))
                 };
                 for c in self.inner.borrow().borrow().components.iter() {
-                    c.1.as_ref().map(|x| {
+                    if let Some(x) = c.1.as_ref() {
                         get_height_visitor(x.as_pin_ref().get_item_ref(0));
-                    });
+                    }
                 }
 
                 let element_height = if count.get() > 0 {
                     total_height.get() / (count.get() as f32)
                 } else {
-                    // There seems to be currently no items. Just instentiate one item.
+                    // There seems to be currently no items. Just instantiate one item.
 
                     {
                         let inner = self.inner.borrow();
@@ -529,9 +649,9 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
 
                     self.ensure_updated_impl(&init, &model, 1);
                     if let Some(c) = self.inner.borrow().borrow().components.get(0) {
-                        c.1.as_ref().map(|x| {
+                        if let Some(x) = c.1.as_ref() {
                             get_height_visitor(x.as_pin_ref().get_item_ref(0));
-                        });
+                        }
                     } else {
                         panic!("Could not determine size of items");
                     }
@@ -561,19 +681,18 @@ impl<C: RepeatedComponent + 'static> Repeater<C> {
                     break;
                 }
             })
-            .is_none()
-        {
-            if self.inner.borrow().borrow().is_dirty.as_ref().get() {
-                let count = self
-                    .inner
-                    .borrow()
-                    .borrow()
-                    .components
-                    .len()
-                    .min(row_count.saturating_sub(self.inner.borrow().borrow().offset));
-                self.ensure_updated_impl(init, &model, count);
-                self.compute_layout_listview(viewport_width, listview_width);
-            }
+            .is_none();
+
+        if geometry_changed && self.inner.borrow().borrow().is_dirty.as_ref().get() {
+            let count = self
+                .inner
+                .borrow()
+                .borrow()
+                .components
+                .len()
+                .min(row_count.saturating_sub(self.inner.borrow().borrow().offset));
+            self.ensure_updated_impl(init, &model, count);
+            self.compute_layout_listview(viewport_width, listview_width);
         }
     }
 
@@ -645,6 +764,11 @@ impl<C: RepeatedComponent> Repeater<C> {
         self.inner.borrow().borrow().components.len()
     }
 
+    /// Return true if the Repeater is empty
+    pub fn is_empty(&self) -> bool {
+        self.len() == 0
+    }
+
     /// Returns a vector containing all components
     pub fn components_vec(&self) -> Vec<ComponentRc<C>> {
         self.inner.borrow().borrow().components.iter().flat_map(|x| x.1.clone()).collect()
@@ -663,13 +787,15 @@ impl<C: RepeatedComponent> Repeater<C> {
         let mut y_offset = inner.offset as f32 * inner.cached_item_height;
         viewport_width.set(listview_width);
         for c in self.inner.borrow().borrow().components.iter() {
-            c.1.as_ref().map(|x| x.as_pin_ref().listview_layout(&mut y_offset, viewport_width));
+            if let Some(x) = c.1.as_ref() {
+                x.as_pin_ref().listview_layout(&mut y_offset, viewport_width);
+            }
         }
         y_offset
     }
 }
 
-/// Represend an item in a StandardListView
+/// Represent an item in a StandardListView
 #[repr(C)]
 #[derive(Clone, Default, Debug, PartialEq)]
 pub struct StandardListViewItem {
