@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -266,7 +266,9 @@ impl DependencyListHead {
     unsafe fn append(_self: *mut Self, node: *const DependencyNode) {
         (*node).debug_assert_valid();
         let old = (*_self).0.get() as *const DependencyNode;
-        old.as_ref().map(|x| x.debug_assert_valid());
+        if let Some(x) = old.as_ref() {
+            x.debug_assert_valid();
+        }
         (*_self).0.set(node as usize);
         let node = &*node;
         node.next.set(old);
@@ -403,12 +405,14 @@ impl PropertyHandle {
 
     /// Implementation of Self::set_binding.
     fn set_binding_impl(&self, binding: *mut BindingHolder) {
-        if self.access(|b| {
+        let previous_binding_intercepted = self.access(|b| {
             b.map_or(false, |b| unsafe {
                 // Safety: b is a BindingHolder<T>
                 (b.vtable.intercept_set_binding)(&*b as *const BindingHolder, binding)
             })
-        }) {
+        });
+
+        if previous_binding_intercepted {
             return;
         }
 
@@ -483,8 +487,8 @@ impl Drop for PropertyHandle {
 }
 
 /// Safety: the dependency list must be valid and consistent
-unsafe fn mark_dependencies_dirty(deps: *mut DependencyListHead) {
-    let mut next = (*deps).0.get() as *const DependencyNode;
+unsafe fn mark_dependencies_dirty(dependencies: *mut DependencyListHead) {
+    let mut next = (*dependencies).0.get() as *const DependencyNode;
     while let Some(node) = next.as_ref() {
         node.debug_assert_valid();
         next = node.next.get();
@@ -498,7 +502,7 @@ unsafe fn mark_dependencies_dirty(deps: *mut DependencyListHead) {
 /// Types that can be set as bindings for a Property<T>
 pub trait Binding<T> {
     /// Evaluate the binding and return the new value
-    fn evaluate(self: &Self, old_value: &T) -> T;
+    fn evaluate(&self, old_value: &T) -> T;
 }
 
 impl<T, F: Fn() -> T> Binding<T> for F {
@@ -606,21 +610,24 @@ impl<T: Clone> Property<T> {
     where
         T: PartialEq,
     {
-        if !self.handle.access(|b| {
+        let previous_binding_intercepted = self.handle.access(|b| {
             b.map_or(false, |b| unsafe {
                 // Safety: b is a BindingHolder<T>
                 (b.vtable.intercept_set)(&*b as *const BindingHolder, &t as *const T as *const ())
             })
-        }) {
+        });
+        if !previous_binding_intercepted {
             self.handle.remove_binding();
         }
+
         // Safety: PropertyHandle::access ensure that the value is locked
-        if self.handle.access(|_| unsafe {
+        let has_value_changed = self.handle.access(|_| unsafe {
             *self.value.get() != t && {
                 *self.value.get() = t;
                 true
             }
-        }) {
+        });
+        if has_value_changed {
             self.handle.mark_dirty();
         }
     }
@@ -727,7 +734,7 @@ impl<T: Clone + InterpolatedPropertyValue + 'static> Property<T> {
             compute_animation_details: || -> AnimationDetail { None },
         };
 
-        // Safety: the AnimatedBindingCallable's type match the property type
+        // Safety: the `AnimatedBindingCallable`'s type match the property type
         unsafe { self.handle.set_binding(binding_callable) };
         self.handle.mark_dirty();
     }
@@ -760,7 +767,7 @@ impl<T: Clone + InterpolatedPropertyValue + 'static> Property<T> {
             compute_animation_details: move || Some(compute_animation_details()),
         };
 
-        // Safety: the AnimatedBindingCallable's type match the property type
+        // Safety: the `AnimatedBindingCallable`'s type match the property type
         unsafe { self.handle.set_binding(binding_callable) };
         self.handle.mark_dirty();
     }
@@ -845,7 +852,7 @@ impl<T: PartialEq + Clone + 'static> Property<T> {
         // Safety: TwoWayBinding's T is the same as the type for both properties
         unsafe {
             prop1.handle.set_binding(TwoWayBinding { common_property: common_property.clone() });
-            prop2.handle.set_binding(TwoWayBinding { common_property: common_property.clone() });
+            prop2.handle.set_binding(TwoWayBinding { common_property });
         }
         prop1.handle.mark_dirty();
     }
@@ -923,7 +930,7 @@ impl<T: InterpolatedPropertyValue + Clone> PropertyValueAnimationData<T> {
         if time_progress >= duration {
             if self.loop_iteration < self.details.loop_count || self.details.loop_count < 0 {
                 self.loop_iteration += (time_progress / duration) as i32;
-                time_progress = time_progress % duration;
+                time_progress %= duration;
                 self.start_time = new_tick - std::time::Duration::from_millis(time_progress as u64);
             } else {
                 return (self.to_value.clone(), true);
@@ -1389,7 +1396,7 @@ pub struct PropertyTracker<ChangeHandler = ()> {
 
 impl Default for PropertyTracker<()> {
     fn default() -> Self {
-        static VT: &'static BindingVTable = &BindingVTable {
+        static VT: &BindingVTable = &BindingVTable {
             drop: |_| (),
             evaluate: |_, _| BindingResult::KeepBinding,
             mark_dirty: |_, _| (),
@@ -1623,6 +1630,7 @@ fn test_nested_property_tracker_dirty() {
 }
 
 #[test]
+#[allow(clippy::redundant_closure)]
 fn test_nested_property_tracker_evaluate_if_dirty() {
     let outer_tracker = Box::pin(PropertyTracker::default());
     let inner_tracker = Box::pin(PropertyTracker::default());
