@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -42,10 +42,12 @@ declare_ValueType![
     crate::items::TextWrap,
     crate::model::StandardListViewItem,
     crate::items::ImageFit,
+    crate::items::ImageRendering,
     crate::input::KeyEvent,
     crate::items::EventResult,
     crate::Brush,
     crate::items::FillRule,
+    crate::graphics::Point,
 ];
 
 /// What kind of animation is on a binding
@@ -99,7 +101,8 @@ pub trait PropertyInfo<Item, Value> {
 
     /// Calls Property::link_two_ways with the property represented here and the property pointer
     ///
-    /// Safety: the property2 must be a pinned pointer to a Property of the same type
+    /// # Safety
+    /// the property2 must be a pinned pointer to a Property of the same type
     #[allow(unsafe_code)]
     unsafe fn link_two_ways(&self, item: Pin<&Item>, property2: *const ());
 }
