@@ -1,7 +1,7 @@
 /* LICENSE BEGIN
     This file is part of the SixtyFPS Project -- https://sixtyfps.io
-    Copyright (c) 2020 Olivier Goffart <olivier.goffart@sixtyfps.io>
-    Copyright (c) 2020 Simon Hausmann <simon.hausmann@sixtyfps.io>
+    Copyright (c) 2021 Olivier Goffart <olivier.goffart@sixtyfps.io>
+    Copyright (c) 2021 Simon Hausmann <simon.hausmann@sixtyfps.io>
 
     SPDX-License-Identifier: GPL-3.0-only
     This file is also available under commercial licensing terms.
@@ -12,7 +12,7 @@ LICENSE END */
 #![allow(unsafe_code)]
 
 use crate::input::{KeyEvent, KeyEventType, KeyboardModifiers, MouseEvent};
-use crate::window::ComponentWindow;
+use crate::window::WindowRc;
 use crate::SharedString;
 
 /// SixtyFPS animations do not use real time, but use a mocked time.
@@ -34,7 +34,7 @@ pub extern "C" fn sixtyfps_send_mouse_click(
     component: &crate::component::ComponentRc,
     x: f32,
     y: f32,
-    window: &ComponentWindow,
+    window: &WindowRc,
 ) {
     let mut state = crate::input::MouseInputState::default();
     let pos = euclid::point2(x, y);
@@ -65,7 +65,7 @@ pub extern "C" fn sixtyfps_send_mouse_click(
 pub extern "C" fn send_keyboard_string_sequence(
     sequence: &crate::SharedString,
     modifiers: KeyboardModifiers,
-    window: &ComponentWindow,
+    window: &WindowRc,
 ) {
     for ch in sequence.chars() {
         let mut modifiers = modifiers;
@@ -74,12 +74,12 @@ pub extern "C" fn send_keyboard_string_sequence(
         }
         let text: SharedString = ch.to_string().into();
 
-        window.process_key_input(&KeyEvent {
+        window.clone().process_key_input(&KeyEvent {
             event_type: KeyEventType::KeyPressed,
             text: text.clone(),
             modifiers,
         });
-        window.process_key_input(&KeyEvent {
+        window.clone().process_key_input(&KeyEvent {
             event_type: KeyEventType::KeyReleased,
             text,
             modifiers,
@@ -102,6 +102,8 @@ cfg_if::cfg_if! {
             ($($t:tt)*) => ($crate::tests::log(&format_args!($($t)*).to_string()))
         }
     } else {
+        /// This macro allows producing debug output that will appear on stderr in regular builds
+        /// and in the console log for wasm builds.
         #[macro_export]
         macro_rules! debug_log {
             ($($t:tt)*) => (eprintln!($($t)*))
